Package: fahb
Title: Design and Analysis of Pilot Trials Assessing Recruitment
        Feasibility
Version: 1.0.0
Authors@R: 
    person("Duncan", "Wilson", , "d.t.wilson@leeds.ac.uk", 
    role = c("aut", "cre", "cph"), comment = c(ORCID = "0000-0001-7949-8718"))
Description: Find optimal decisions rules for guiding progression decisions
    following a pilot trial, assuming a hierarchical recruitment model. Estimate
    the time until the main trial recruits to target, given the recruitment data
    observed in the pilot.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: BH, knitr, patchwork, RcppEigen, rmarkdown, testthat (>=
        3.0.0)
Config/testthat/edition: 3
Imports: brms, ggplot2, mco, mgcv, posterior, rlang
VignetteBuilder: knitr
URL: https://dtwilson.github.io/fahb/, https://github.com/DTWilson/fahb
BugReports: https://github.com/DTWilson/fahb/issues
NeedsCompilation: no
Packaged: 2026-04-28 15:01:50 UTC; meddwilb
Author: Duncan Wilson [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-7949-8718>)
Maintainer: Duncan Wilson <d.t.wilson@leeds.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-29 08:00:02 UTC
Built: R 4.5.2; ; 2026-04-29 10:48:51 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,fahb_analysis)
S3method(plot,fahb_design)
S3method(print,fahb_analysis)
S3method(print,fahb_design)
export(check_priors)
export(fahb_analysis)
export(fahb_design)
export(fahb_problem)
export(forecast)
importFrom(rlang,.data)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# fahb <a href="https://dtwilson.github.io/fahb/"><img src="man/figures/logo.png" align="right" height="139" alt="tout website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/DTWilson/fahb/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/DTWilson/fahb/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/DTWilson/fahb//branch/main/graph/badge.svg)](https://app.codecov.io/gh/DTWilson/fahb)
<!-- badges: end -->

The goal of `fahb` is to help with clinical trial **f**easibility
**a**ssessment via **h**ierarchical **B**ayesian recruitment models,
fitted to (internal or external) pilot date. It can be used at the
design stage to optimise the pilot and any pre-specified decision rules
to guide progression, and at the analysis stage to produce a
probabilistic prediction of the time until the main trial recruits to
target.

## Installation

Install the released version of `fahb` from CRAN:

``` r
install.packages("fahb")
```

Or you can install the development version of `fahb` from
[GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("DTWilson/fahb")
```

## Example

Suppose we are planning an internal pilot for a trial which aims to
recruit `N = 320` participants from `m = 20` sites, and that we expect
recruitment to complete in three years. The pilot analysis will happen
at `t = 0.5` years, and we would class the trial as infeasible if
recruitment took longer than `rel_thr = 1.2` times the expected
recruitment time.

The variables `N, m, t` and `rel_thr` define the design variables. For
the model parameters, for the purpose of illustration we use the package
defaults (but see the vignette and the associated paper for more
discussion of these). To use `fahb` we first encode all design variables
and model parameters in a `fahb_problem` object, then run some
simulations via `forecast()` before finding possible decision rules and
the operating characteristics they lead to:

``` r
library(fahb)

problem <- fahb_problem(N = 320, m = 20, t = 0.5, rel_thr = 1.2)

# Run the simulations
problem <- forecast(problem)

# Find some candidate decision rules are their OCs
design <- fahb_design(problem)

print(design)
#> Standard progression criteria
#> 
#>     FPR        FNR         n_p          m_p        r_p
#> 1   0.0 0.82817058 17.81878829 -0.059458005  9.6590446
#> 11  0.1 0.45825809  9.57670582  1.334793215  6.6483566
#> 21  0.2 0.31559939  6.90517099  0.182386148  6.0045943
#> 41  0.4 0.18030282  1.59558082  0.759740721  5.6010379
#> 51  0.5 0.12835116  2.70338476  1.696370342  4.5998899
#> 71  0.7 0.05625781  0.14306472  0.904175281  3.5567570
#> 81  0.8 0.03681067 -0.30186503  0.937890251  2.8416713
#> 91  0.9 0.02222531 -0.90403448 -0.022824150  1.9307604
#> 101 1.0 0.00000000  0.01221748  0.008838253 -0.7620012
#> 
#> Bayesian approximation
#> 
#>     FPR        FNR      T_p
#> 1   0.0 1.00000000 1.441843
#> 11  0.1 0.42366995 3.123216
#> 21  0.2 0.31309904 3.298833
#> 41  0.4 0.16335602 3.594781
#> 51  0.5 0.11543270 3.721615
#> 71  0.7 0.05723017 3.916746
#> 81  0.8 0.03653285 4.001302
#> 91  0.9 0.02139186 4.085859
#> 101 1.0 0.00000000 4.554172
#> 
#> FPR - False Positive Rate
#> FNR - False Negative Rate
#> 
#> n_p, m_p, r_p - Probabilistic thresholds for standard
#>                 progression criteria on the number recruited,
#>                 number of sites opened, and the recruitment rate
#>                 (participants per site per year) respectively
#> 
#> T_p - Bayesian decision rule threshold for the posterior predictive
#>       expected time until full recruitment
plot(design)
```

<img src="man/figures/README-unnamed-chunk-2-1.png" alt="" width="100%" />

Two types of decision rules are considered here. Firstly, we consider
rules which take the same form as the standard progression criteria
suggested by the NIHR. In the table output, `n_p` is a minimum number of
participants recruited, `m_p` is a minimum number of sites opened, and
`r_p` is a minimum rate of recruitment (participants per site-year), and
we progress to the main trial only if all of these thresholds are met.

The second type of decision rules is defined by `T_p`, the maximum
expected time until full recruitment conditional on the pilot data. We
progress to the main trial only if the actual posterior predictive
expectation is lower than this threshold.

Decision rules of both types are characterised by their false positive
(`FPR`) and false negative (`FNR`) rates. These are estimated via
simulation by comparing the decisions made with the underlying
feasibility, as determined by the threshold `rel_thr`. For example, we
can attain `FPR = 0.15` and `FNR = 0.34` if we proceed only if we
recruit at least $10.48$ participants from at least $1$ site, with an
overall rate of at least $6.57$ participants per site-year. We can get
slightly better operating characteristics if we use a Bayesian
progression rules instead. If we want to improve the pilot further we
can increase the design variable `t_int` so that the internal pilot will
have more data.

#' Plot prior distributions
#' 
#' Given the hyperparameters encoded in a `fahb_problem` object, return plots of
#' the three prior distributions and a plot of the prior predictive distribution
#' of the recruitment rate at a randomly selected site.
#'
#' @param problem an object of class `fahb_problem`.
#'
#' @returns a list of `ggplot2` plots.
#' @export
#'
#' @examples
#' problem <- fahb_problem()
#' check_priors(problem)
#' 
check_priors <- function(problem){
  # Plot prior densities for the three model parameters, and the prior 
  # predictive densities for the recruitment rate at a random site and
  # the time needed to fully recruit
  p_so <- plot_gamma_prior(problem$so_hp_a, problem$so_hp_b, "Site opening rate")
  p_mean_rr <- plot_normal_prior(problem$mean_rr_hp_a, problem$mean_rr_hp_b, 
                                  "Participant recruitment rate (LogNormal mean)")
  p_sd_rr <- plot_gamma_prior(problem$sd_rr_hp_a, problem$sd_rr_hp_b, 
                              "Participant recruitment rate (LogNormal SD)")
  p_pp_rr <- plot_rr_prior_predictive(problem)
  #p_pp_t <- plot_t_prior_predictive(problem)
  
  return(list(p_so,
              p_mean_rr,
              p_sd_rr,
              p_pp_rr))
}

plot_gamma_prior <- function(shape, rate, par_name, post = FALSE){
  
  lower <- max(0, shape/rate - 4*sqrt(shape/rate^2))
  upper <- shape/rate + 4*sqrt(shape/rate^2)
  
  x <- seq(lower, upper, length.out = 10^3)
  df <- data.frame(x = x,
                   pr = stats::dgamma(x, shape, rate=rate))
  
  p <- ggplot2::ggplot(df, ggplot2::aes(.data$x, .data$pr)) + ggplot2::geom_line() +
    ggplot2::xlab(par_name) +
    ggplot2::theme_minimal()
  if(post){ 
    p <- p + ggplot2::ylab("Posterior prob.")
  } else {
    p <- p + ggplot2::ylab("Prior prob.")
  }
    
  return(p)
}

plot_normal_prior <- function(mean, sd, par_name){
  
  lower <- mean - 4*sd
  upper <- mean + 4*sd
  
  x <- seq(lower, upper, length.out = 10^3)
  df <- data.frame(x = x,
                   pr = stats::dnorm(x, mean, sd))
  
  p <- ggplot2::ggplot(df, ggplot2::aes(.data$x, .data$pr)) + ggplot2::geom_line() +
    ggplot2::xlab(par_name) + ggplot2::ylab("Prior prob.") +
    ggplot2::theme_minimal()
  
  return(p)
}

plot_rr_prior_predictive <- function(problem){
  df <- data.frame(beta = stats::rnorm(10^5, problem$mean_rr_hp_a, problem$mean_rr_hp_b),
                   sig = stats::rgamma(10^5, problem$sd_rr_hp_a, rate = problem$sd_rr_hp_b))
  df$gamma <- exp(stats::rnorm(10^5, df$beta, df$sig))
  
  p <- ggplot2::ggplot(df, ggplot2::aes(.data$gamma)) + 
    ggplot2::stat_density(alpha = 0, colour = "black") +
    ggplot2::xlab("Participant recruitment rate at random site") +
    ggplot2::ylab("Prior pred. prob.") +
    ggplot2::xlim(as.numeric(stats::quantile(df$gamma, c(0.001, 0.999)))) +
    ggplot2::theme_minimal()
  
  return(p)
}
  
  

bayesian_ocs <- function(problem){
  if(is.null(problem$sims)){
    stop("Simulations are missing - run forecast()")
  }
  
  df <- problem$sims
  unique_m_ps <- length(unique(df$m_p))
  if(unique_m_ps >= 5){
    fit <- mgcv::gam(rec_T ~ ti(n_p) + ti(m_p) + ti(r_p) + ti(m_p, n_p), data = df)
  } else {
    fit <- mgcv::gam(rec_T ~ ti(n_p) + ti(r_p), data = df)
  }
  df$pred_T <- mgcv::predict.gam(fit)
  
  rules <- seq(min(df$pred_T), max(df$pred_T), length.out = 1000)
  vals <- t(sapply(rules, Bayes_OCs, thr = problem$thr, df = df))
  opt_vals <- cbind(vals, rules)
  
  ## Reduce down to a summary
  opt_vals <- opt_vals[order(-opt_vals[,1]),]
  
  nom_fprs <- seq(0, 1, 0.01)
  opt_vals <- t(sapply(nom_fprs, Bayes_opt_fnr, opt_vals = opt_vals))
  opt_vals[,1] <- nom_fprs
  
  opt_vals <- as.data.frame(opt_vals)
  names(opt_vals) <- c("FPR", "FNR", "T_p")
  
  return(opt_vals)
}

Bayes_OCs <- function(rule, thr, df){
  # Calculate FPR and FNR for a given Bayes rule
  
  feas <- df$rec_T < thr
  go <- !((df$pred_T >= rule) | (df$pred_T < 0))
  
  fpr <- sum(go[!feas])/(sum(!feas))
  fnr <- sum(!go[feas])/(sum(feas))
  
  return(c(fpr, fnr))
}

Bayes_opt_fnr <- function(nom_fpr, opt_vals){
  # Find the best FNR for a nominal FPR
  return(opt_vals[opt_vals[,1] <= nom_fpr, ,drop = FALSE][1,])
}

test_that("bayesian_ocs errors if simulations are missing", {
  problem <- fahb_problem()
  
  expect_error(
    bayesian_ocs(problem),
    "Simulations"
  )
})

test_that("bayesian_ocs returns a data frame with expected columns", {
  set.seed(123)
  
  problem <- fahb_problem() |>
    forecast(n_sims = 50)
  
  ocs <- bayesian_ocs(problem)
  
  expect_true(is.data.frame(ocs))
  expect_named(ocs, c("FPR", "FNR", "T_p"))
})

test_that("bayesian_ocs returns sensible operating characteristics", {
  set.seed(123)
  
  problem <- fahb_problem() |>
    forecast(n_sims = 50)
  
  ocs <- bayesian_ocs(problem)
  
  expect_true(all(is.finite(ocs$FPR)))
  expect_true(all(is.finite(ocs$FNR)))
  expect_true(all(is.finite(ocs$T_p)))
  
  expect_true(all(ocs$FPR >= 0 & ocs$FPR <= 1))
  expect_true(all(ocs$FNR >= 0 & ocs$FNR <= 1))
})

test_that("bayesian_ocs nominal FPR grid is increasing", {
  set.seed(123)
  
  problem <- fahb_problem() |>
    forecast(n_sims = 50)
  
  ocs <- bayesian_ocs(problem)
  
  expect_true(all(diff(ocs$FPR) >= 0))
})


test_that("Bayes_OCs returns FPR and FNR", {
  df <- data.frame(
    rec_T  = c(1, 2, 3, 4),
    pred_T = c(1.5, 2.5, 0.5, 1.0)
  )
  
  thr <- 2
  rule <- 1.5
  
  out <- Bayes_OCs(rule, thr, df)
  
  expect_type(out, "double")
  expect_length(out, 2)
})

test_that("Bayes_OCs returns values in [0,1]", {
  df <- data.frame(
    rec_T  = c(1, 2, 3, 4),
    pred_T = c(1.5, 2.5, 0.5, 1.0)
  )
  
  thr <- 2
  rule <- 1.5
  
  out <- Bayes_OCs(rule, thr, df)
  
  expect_true(all(out >= 0 & out <= 1))
})

test_that("Bayes_opt_fnr returns best FNR under nominal FPR", {
  opt_vals <- matrix(
    c(
      0.10, 0.40, 1.0,
      0.20, 0.30, 1.5,
      0.30, 0.20, 2.0
    ),
    ncol = 3,
    byrow = TRUE
  )
  
  res <- Bayes_opt_fnr(0.2, opt_vals)
  
  expect_equal(res[1], 0.10)
  expect_equal(res[2], 0.40)
})

test_that("Bayes_opt_fnr respects FPR constraint", {
  opt_vals <- matrix(
    c(
      0.05, 0.50, 1.0,
      0.10, 0.30, 1.5,
      0.20, 0.10, 2.0
    ),
    ncol = 3,
    byrow = TRUE
  )
  
  res <- Bayes_opt_fnr(0.1, opt_vals)
  
  expect_lte(res[1], 0.1)
})

new_fahb_design <- function(problem,
                            PC_OCs, Bayes_OCs){
  structure(list(problem = problem,
                 Prog_Crit_OCs = PC_OCs,
                 Bayes_OCs = Bayes_OCs),
            class = "fahb_design")
}

#' Build a `fahb` design object
#' 
#' Given a `fahb_problem` object, find efficient progression decision rules. 
#' These can include rules of the standard "progression criteria form", or rules
#' based on a Bayesian analysis of the pilot trial data, or both. 
#'
#' @param problem an object of class `fahb_problem`.
#' @param quietly if this argument is set to FASLE then information about which
#' steps have been completed will be printed to the console. Defaults to TRUE.
#'
#' @returns an object of class `fahb_design`.
#' @export
#'
#' @examples
#' problem <- forecast(fahb_problem(), n_sims = 500)
#' fahb_design(problem)
#' 
fahb_design <- function(problem, quietly = TRUE){
  if(is.null(problem$sims)){
    stop("Problem does not contain trial simulations - see fahb::forecast()")
  }
  
  if(!quietly){
    message("Searching for efficient progression criteria...\n")
  }
  PC_OCs <- prog_crit_ocs(problem)

  if(!quietly){
    message("Approximating Bayesian operating characteristics...\n")
  }
  Bayes_OCs <- bayesian_ocs(problem)

  new_fahb_design(problem,
                  PC_OCs, Bayes_OCs)
}


#' Print a fahb design object
#' 
#' The default print method for a `fahb_design` object.
#' 
#' @param x object of class `fahb_design` as produced by `fahb_design()`.
#' @param coarse binary indicator that only a coarse subset of all decision 
#' rules should be printed. Defaults to TRUE.
#' @param ... further arguments passed to or from other methods.
#' 
#' @return no return value, called for side effects.
#' 
#' @export
print.fahb_design <- function(x, coarse = TRUE, ...){
  cat("Standard progression criteria\n")
  cat("\n")
  if(coarse){
    print(x$Prog_Crit_OCs[x$Prog_Crit_OCs$FPR %in% seq(0,1,0.1),])
  } else {
    print(x$Prog_Crit_OCs)
  }
  cat("\n")
  
  cat("Bayesian approximation\n")
  cat("\n")
  if(coarse){
    print(x$Bayes_OCs[x$Bayes_OCs$FPR %in% seq(0,1,0.1),])
  } else {
    print(x$Bayes_OCs)
  }
  cat("\n")
  cat("FPR - False Positive Rate\n")
  cat("FNR - False Negative Rate\n\n")
  cat("n_p, m_p, r_p - Probabilistic thresholds for standard\n")
  cat("                progression criteria on the number recruited,\n")
  cat("                number of sites opened, and the recruitment rate\n")
  cat("                (participants per site per year) respectively\n\n")
  cat("T_p - Bayesian decision rule threshold for the posterior predictive\n")
  cat("      expected time until full recruitment\n")
}


#' Plot operating characteristics of fahb designs
#' 
#' Takes an object of class `fahb_design` and plots the estimated operating 
#' characteristics of decision rules - based on standard progression criteria,
#' an approximate Bayesian analysis, or both.
#' 
#' @param x object of class `fahb_design` as produced by `fahb_design().`
#' @param ... further arguments passed to or from other methods.
#' 
#' @return no return value, called for side effects.
#' 
#' @export
plot.fahb_design <- function(x, ...){
  PC_OCs <- x$Prog_Crit_OCs[,1:2]
  PC_OCs$type <- "PC"
  Bayes_OCs <- x$Bayes_OCs[,1:2]
  Bayes_OCs$type <- "Bayes"
  
  all_OCs <- rbind(PC_OCs, Bayes_OCs)
  
  p <- ggplot2::ggplot(all_OCs, ggplot2::aes(.data$FPR, .data$FNR, colour = .data$type)) + ggplot2::geom_step() +
    ggplot2::scale_color_discrete(name = "Method") +
    ggplot2::xlab("False Positive Rate") +
    ggplot2::ylab("False Negative Rate") +
    ggplot2::theme_minimal()
  
  return(p)
}

test_that("fahb_design errors if simulations are missing", {
  problem <- fahb_problem()
  
  expect_error(
    fahb_design(problem),
    "does not contain trial simulations"
  )
})

test_that("fahb_design returns a fahb_design object with problem included", {
  set.seed(123)
  
  problem <- fahb_problem()
  problem <- forecast(problem, n_sims = 20)
  
  design <- fahb_design(problem)
  
  expect_s3_class(design, "fahb_design")
  expect_type(design, "list")
  
  expect_named(
    design,
    c("problem", "Prog_Crit_OCs", "Bayes_OCs")
  )
})

test_that("fahb_design stores the original problem object", {
  set.seed(123)
  
  problem <- fahb_problem()
  problem <- forecast(problem, n_sims = 20)
  
  design <- fahb_design(problem)
  
  expect_identical(design$problem, problem)
})

test_that("fahb_design components are data frames", {
  set.seed(123)
  
  problem <- fahb_problem()
  problem <- forecast(problem, n_sims = 20)
  
  design <- fahb_design(problem)
  
  expect_true(is.data.frame(design$Prog_Crit_OCs))
  expect_true(is.data.frame(design$Bayes_OCs))
})

test_that("print.fahb_design produces output without error", {
  set.seed(123)
  
  problem <- fahb_problem() |>
    forecast(n_sims = 20)
  
  design <- fahb_design(problem)
  
  expect_output(
    print(design),
    "Standard progression criteria"
  )
})

test_that("plot.fahb_design returns a ggplot object", {
  set.seed(123)
  
  problem <- fahb_problem() |>
    forecast(n_sims = 20)
  
  design <- fahb_design(problem)
  
  p <- plot(design)
  
  expect_s3_class(p, "ggplot")
})

test_that("plot.fahb_design includes PC and Bayesian methods", {
  set.seed(123)
  
  problem <- fahb_problem() |>
    forecast(n_sims = 20)
  
  design <- fahb_design(problem)
  
  p <- plot(design)
  plot_data <- p$data
  
  expect_true(all(c("PC", "Bayes") %in% plot_data$type))
})

new_fahb_analysis <- function(PC_stats, Bayes_stats,
                              PP_rec_times, so_post_hps,
                              model_fit){
  structure(list(PC_stats = PC_stats, Bayes_stats = Bayes_stats,
                 PP_rec_times = PP_rec_times, so_post_hps = so_post_hps,
                 model_fit = model_fit),
            class = "fahb_analysis")
}

#' Build a `fahb` analysis object
#' 
#' Given a `fahb_problem` object calculate summary statistics which can inform
#' progression decisions. These include both standard progression criteria 
#' statistics, and the expectation of the posterior predictive distribution of 
#' the time until the trial recruits.
#'
#' @param n_pilot integer vector of numbers recruited at each open site.
#' @param t_pilot numeric vector of time (in years) each site has been open.
#' @param problem object of class `fahb_problem`.
#' @param site_t In the case of an external pilot, the time taken for all 
#' pilot sites to open.
#' @param bayes_model optional object of class `brmsfit` which will be used in
#' the Bayesian analysis via `brms::update()` to avoid compiling a new model.
#'
#' @returns An object of class `fahb_analysis`.
#' @export
#'
#' @examples
#' ## Example illustrating a full analysis workflow
#' ## (Not run on CRAN due to Bayesian model fitting)
#'
#' \donttest{
#' problem <- fahb_problem()
#' problem <- forecast(problem)
#'
#' ## Pilot trial data
#' n_pilot <- c(3, 5, 2)
#' t_pilot <- c(0.5, 0.6, 0.4)
#'
#' analysis <- fahb_analysis(
#'   n_pilot = n_pilot,
#'   t_pilot = t_pilot,
#'   problem = problem
#' )
#'
#' print(analysis)
#' plot(analysis)
#' }
fahb_analysis <- function(n_pilot, t_pilot, 
                          problem, 
                          site_t = NULL,
                          bayes_model = NULL){
  
  if(is.null(site_t)){
    site_t <- problem$t
    if(!problem$internal){
      stop("External pilots require site_t to be specified")
    }
  }
  
  if(length(n_pilot) != length(t_pilot)){
    stop("Lengths of n_pilot and t_pilot must match")
  }
  
  m <- problem$m
  
  # Get standard PC summaries
  n_p <- sum(n_pilot)
  m_p <- length(n_pilot)
  r_p <- n_p/sum(t_pilot)
  
  # Fit a Bayesian model
  if(is.null(bayes_model)){
    message("Compiling the model...")
    bayes_model <- compile_bayes_model(n_pilot, t_pilot, problem)
  }
  
  int_data <- data.frame(y = n_pilot,
                         t = t_pilot,
                         c = 1:length(n_pilot))
  
  # Generate posterior samples
  output <- utils::capture.output(fit <- stats::update(bayes_model, 
                                                          backend = "cmdstanr",
                                                          recompile = FALSE, 
                                                          newdata = int_data, 
                                                          iter = 3000, 
                                                          warmup = 500,
                                                          control = list(adapt_delta = 0.95)))
  
  # Get vectors of posterior samples for the mean and SD parameters of the 
  # log-normal model for site recruitment rates
  s <- brms::as_draws(fit)
  beta_0 <- posterior::extract_variable(s, "b_Intercept")
  sd_r <- posterior::extract_variable(s, "sd_c__Intercept")
  
  # First deal with cases where not all sites have yet opened
  if(m - m_p > 0){
    
    # Generate posterior samples of recruitment rates at all future sites,
    # Using the posterior samples of the log-normal recruitment rate model.
    # Each column of u is a yearly rate at that site, each row a sample from the 
    # posterior.
    us <- sapply(1:(m-m_p), function(x) exp(stats::rnorm(length(beta_0), beta_0, sd_r)))
    
    # Add to that matrix the posterior samples for the sites which have already
    # opened
    r <- brms::ranef(fit, summary = F)
    us <- cbind(us, exp(r$c[,1:m_p,1] + beta_0))
    
    # For site setup rates we have a simple conjugate analysis
    setup_r_a1 <- problem$so_hp_a + m_p
    setup_r_b1 <- problem$so_hp_b + site_t
    
    # We can then generate samples from the posterior of setup rate
    setup_rates <- stats::rgamma(nrow(us), setup_r_a1, setup_r_b1)
    
    if(problem$internal){
      # Now sample actual site setup times from the posterior predictive
      setup_times <- t(sapply(setup_rates, function(x) cumsum(stats::rexp(m-m_p, x))))
      if(m-m_p == 1) setup_times <- t(setup_times)
      
      # Add pilot sites using a setup time of 0
      setup_times <- cbind(setup_times, matrix(rep(0, m_p*nrow(us)), ncol = m_p))
      
      site_matrix <- cbind(us, setup_times)
      
      rec_times <- apply(site_matrix, 1, post_pred_rec_time, 
                         m=m, target_n=(problem$N - n_p)) + problem$t
    } else {
      # For external, we want setup times for all m sites
      setup_times <- t(sapply(setup_rates, function(x) cumsum(stats::rexp(m, x))))
      
      site_matrix <- cbind(us, setup_times)
      
      rec_times <- apply(site_matrix, 1, post_pred_rec_time, 
                         m=m, target_n=problem$N)
    }
    
  } else {
    # Case where all sites in the main have also opened in the pilot
    # Get the posterior samples for the sites which have already
    # opened (i.e. all sites)
    r <- brms::ranef(fit, summary = F)
    us <- exp(r$c[,1:m_p,1] + beta_0)
    
    if(problem$internal){
      # Add pilot sites using a setup time of 0
      setup_times <- matrix(rep(0, m_p*nrow(us)), ncol = m_p)
      
      site_matrix <- cbind(us, setup_times)
      
      rec_times <- apply(site_matrix, 1, post_pred_rec_time, 
                         m=m, target_n=(problem$N - n_p)) + problem$t
    } else {
      # For external, we want setup times for all m sites
      setup_times <- t(sapply(setup_rates, function(x) cumsum(stats::rexp(m, x))))
      
      site_matrix <- cbind(us, setup_times)
      
      rec_times <- apply(site_matrix, 1, post_pred_rec_time, 
                         m=m, target_n=problem$N)
    }
  }
  
  new_fahb_analysis(c(n_p=n_p, m_p=m_p, r_p=r_p), c(exp_pp_T=mean(rec_times)),
                    rec_times, so_post_hps = c(setup_r_a1, setup_r_b1),
                    fit)
}

post_pred_rec_time <- function(post_samples, m, target_n){
  # post_samples is a vector of one draw from the posterior distribution.
  # The first m elements are recruitment rates for the m sites; the next m 
  # elements are the corresponding opening times, relative to the interim 
  # analysis point.
  # Returns a sample time to hit recruitment target from the posterior
  # predictive distribution.
  # Note- target_n is number needed to recruit in addiiton to those
  # recruited by interim
  
  end_rec <- 1000
  
  # Split up into the site rates and site setup times
  us <- post_samples[1:m]; setup_ts <- post_samples[(m+1):(2*m)]
  # Re-order each vector in order of site setup times
  us <- us[order(setup_ts)]; setup_ts <- setup_ts[order(setup_ts)]
  
  #################
  # Note - duplicating elements from the DGM function, need to tidy
  #################
  
  # Build a matrix of the recruitment periods with their start times, end
  # times, and overall recruitment rates
  rec_rates <- matrix(c(0, setup_ts,
                        setup_ts, end_rec,
                        0, cumsum(us)), ncol = 3)
  
  # Simulate numbers recruited in each period and add these as a column
  ## First get the vector of expected numbers recruited in each period
  exp_rec <- (rec_rates[,3]*(rec_rates[,2] - rec_rates[,1]))[2:(m+1)]
  ## Now sample from a Poisson for each period
  rec_rates <- cbind(rec_rates, c(0, stats::rpois(m, exp_rec)))
  
  # Get time at which total target n is hit
  ## First get the period when it happens
  fin_period <- which(cumsum(rec_rates[,4]) > target_n)[1]
  ## Determine how many people need to arrive in that final period
  n_needed <- target_n  -  sum(rec_rates[1:(fin_period-1), 4])
  ## Simulate when the target is hit, which follows a gamma distribution
  rec_time <- rec_rates[fin_period, 1] + stats::rgamma(1, shape = n_needed, rate = rec_rates[fin_period, 3])
  
  return(rec_time)
}

compile_bayes_model <- function(n_pilot, t_pilot,
                                problem){
  
  beta_m <- problem$mean_rr_hp_a; beta_s <- problem$mean_rr_hp_b
  v_sh <- problem$sd_rr_hp_a; v_r <- problem$sd_rr_hp_b
  setup_r_a <- problem$so_hp_a; setup_r_b <- problem$so_hp_b
  target_n <- problem$N; m <- problem$m; int_t <- problem$t
  
  stanvars <- brms::stanvar(beta_m, name='beta_m') + 
    brms::stanvar(beta_s, name='beta_s') + 
    brms::stanvar(v_sh, name='v_sh') + brms::stanvar(v_r, name='v_r')
  
  bprior <- c(
    brms::set_prior(
      paste0("normal(", beta_m, ", ", beta_s, ")"), 
      class = "Intercept"),
    brms::set_prior(
      paste0("gamma(", v_sh, ", ", v_r, ")"),
      class = "sd"))
    
  int_data <- data.frame(y = n_pilot,
                     t = t_pilot,
                     c = 1:length(n_pilot))
  
  bayes_model <- brms::brm(y | rate(t) ~ 1 + (1 | c), data = int_data, family = stats::poisson(),
                     prior = bprior, 
                     stanvars = stanvars,
                     chains = 0, silent = 2)
  
  return(bayes_model)
}

#' Print a `fahb` analysis object
#' 
#' The default print method for a `fahb_analysis` object.
#' 
#' @param x object of class `fahb_analysis` as produced by `fahb_analysis()`.
#' @param ... further arguments passed to or from other methods.
#' 
#' @return no return value, called for side effects.
#' 
#' @export
print.fahb_analysis <- function(x, ...){
  cat("Standard progression criteria statistics:\n")
  print(x$PC_stats)
  cat("\n")
  cat("Expected posterior predictive time to recruit:\n")
  print(x$Bayes_stats)
  cat("\n")
  cat("Posterior predictive distribution quantiles:\n")
  print(stats::quantile(x$PP_rec_times, c(0.005, 0.025, 0.2, 0.5, 0.8, 0.975, 0.995)))
  cat("\n")
  cat("Posterior site opening rate hyperparamaters (Gamma):\n")
  print(c(shape = x$so_post_hps[1], rate = x$so_post_hps[2]))
  cat("\n")
}


#' Plot posterior distributions from a `fahb` analysis
#' 
#' Takes an object of class `fahb_analysis` and plots the posterior 
#' distributions of the predicted time for the trial to recruit and of the three
#' model parameters.
#' 
#' @param x object of class `fahb_analysis` as produced by `fahb_analysis().`
#' @param ... further arguments passed to or from other methods.
#' 
#' @return no return value, called for side effects.
#' 
#' @export
plot.fahb_analysis <- function(x, ...){
  
  p1 <- plot_post_samples(x$PP_rec_times, "Predicted time to recruit")
  p2 <- plot_gamma_prior(shape = x$so_post_hps[1], rate = x$so_post_hps[2], 
                         par_name = "Site opening rate", post = TRUE)
  
  s <- brms::as_draws(x$model_fit)
  beta_0 <- posterior::extract_variable(s, "b_Intercept")
  sd_r <- posterior::extract_variable(s, "sd_c__Intercept")
  
  p3 <- plot_post_samples(beta_0, "Mean of log(recruitment rate)")
  p4 <- plot_post_samples(sd_r, "S.D. of log(recruitment rate)")
  
  return(c(p1, p2, p3, p4))
}

plot_post_samples <- function(samples, par_name){
  df <- data.frame(x = samples)
  ggplot2::ggplot(df, ggplot2::aes(x=.data$x)) +
  ggplot2::geom_density() +
    ggplot2::xlab(par_name) +
    ggplot2::ylab("Posterior prob.") +
    ggplot2::xlim(as.numeric(stats::quantile(df$gamma, c(0.0001, 0.9999)))) +
    ggplot2::theme_minimal()
}

test_that("new_fahb_analysis creates a fahb_analysis object", {
  out <- new_fahb_analysis(
    PC_stats = c(n_p = 10, m_p = 2, r_p = 1.5),
    Bayes_stats = c(exp_pp_T = 12.3),
    PP_rec_times = c(10, 11, 12),
    so_post_hps = c(35, 5),
    model_fit = "dummy_model"
  )
  
  expect_s3_class(out, "fahb_analysis")
  expect_type(out, "list")
})

test_that("fahb_analysis object has expected components", {
  out <- new_fahb_analysis(
    PC_stats = c(n_p = 10, m_p = 2, r_p = 1.5),
    Bayes_stats = c(exp_pp_T = 12.3),
    PP_rec_times = c(10, 11, 12),
    so_post_hps = c(35, 5),
    model_fit = "dummy_model"
  )
  
  expect_named(
    out,
    c("PC_stats",
      "Bayes_stats",
      "PP_rec_times",
      "so_post_hps",
      "model_fit")
  )
})

test_that("fahb_analysis errors when pilot inputs are inconsistent", {
  problem <- fahb_problem()
  
  expect_error(
    fahb_analysis(
      n_pilot = c(1, 2),
      t_pilot = c(1),   # length mismatch
      problem = problem
    )
  )
})

test_that("post_pred_rec_time returns a positive finite time", {
  set.seed(123)
  
  m <- 3
  target_n <- 20
  
  post_samples <- c(
    # recruitment rates
    1.2, 1.0, 0.8,
    # setup times
    0, 0.5, 1
  )
  
  t <- post_pred_rec_time(
    post_samples = post_samples,
    m = m,
    target_n = target_n
  )
  
  expect_type(t, "double")
  expect_true(is.finite(t))
  expect_gt(t, 0)
})

test_that("print.fahb_analysis produces output", {
  obj <- new_fahb_analysis(
    PC_stats = c(n_p = 10, m_p = 2, r_p = 1.5),
    Bayes_stats = c(exp_pp_T = 12.3),
    PP_rec_times = c(10, 11, 12),
    so_post_hps = c(35, 5),
    model_fit = "dummy_model"
  )
  
  expect_output(
    print(obj),
    "Standard progression criteria statistics"
  )
})

prog_crit_ocs <- function(problem){
  
  if(is.null(problem$sims)){
    stop("Simulations are missing - run forecast()")
  }
  
  df <- problem$sims
  df <- cbind(df, matrix(stats::runif(2*nrow(df)), ncol=2))
  
  max_m_p <- max(df$m_p)
  max_n_p <- max(df$n_p)
  max_r_p <- max(df$r_p)
  
  opt <- mco::nsga2(PC_OCs, 3, 2,
                    lower.bounds = rep(-1, 3),
                    upper.bounds = c(max_n_p, max_m_p, max_r_p),
                    popsize = 100, generations = 100,
                    thr = problem$thr, df = df)
  
  # Summarise rules by finding the optimal FNR for a series of nominal FPRs
  opt_vals <- cbind(opt$value, opt$par)
  
  ## Reduce down to a summary
  opt_vals <- opt_vals[order(-opt_vals[,1]),]
  
  nom_fprs <- seq(0, 1, 0.01)
  opt_vals <- t(sapply(nom_fprs, PC_opt_fnr, opt_vals = opt_vals))
  opt_vals[,1] <- nom_fprs
  
  opt_vals <- as.data.frame(opt_vals)
  names(opt_vals) <- c("FPR", "FNR", "n_p", "m_p", "r_p")
  
  return(opt_vals)
}

PC_OCs <- function(rule, thr, df){
  # Calculate FPR and FNR for a given rule
  
  feas <- df$rec_T < thr
  #go <- df$n_p >= rule[1] & df$m_p >= rule[2] & df$r_p >= rule[3]
  go <- (df[,5] > (rule[1] - df$n_p)) &
    (df[,6] > (rule[2] - df$m_p)) &
    df$r_p > rule[3]
  
  fpr <- sum(go[!feas])/sum(!feas)
  fnr <- sum(!go[feas])/sum(feas)
  
  return(c(fpr, fnr))
}

PC_opt_fnr <- function(nom_fpr, opt_vals){
  # Find the best FNR for a nominal FPR
  return(opt_vals[opt_vals[,1] <= nom_fpr, ,drop = FALSE][1,])
}

#' fahb: Feasibility Assessment with Hierarchical Bayes
#'
#' @description
#' Tools for simulation, design, and analysis of internal pilot trials
#' using frequentist and Bayesian operating characteristics.
#'
#' @importFrom rlang .data
#'
#' @keywords internal
"_PACKAGE"

simulate_data <- function(beta_m, beta_s,
                          v_sh, v_r,
                          setup_r_a, setup_r_b,
                          target_n, m, 
                          internal,
                          t = t,
                          n_ext = NULL, m_ext = NULL){

  
  int_t <- t
  # True per year recruitment rates for each site
  ## Sample beta_0 from prior
  beta_0 <- stats::rnorm(1, mean = beta_m, sd = beta_s)
  ## Sample var_l from prior
  #var_l <- abs(rlst(1, df = v_df, mu = 0, sigma = v_sc))^2
  sd_l <- stats::rgamma(1, shape = v_sh, rate = v_r)
  ## Sample the site rates from a log-normal 
  lambdas <- exp(stats::rnorm(m, beta_0, sd_l))
  
  # True setup rate, from the prior
  setup_r <- stats::rgamma(1, setup_r_a, setup_r_b)
  
  # Simulate the site openings and participant recruitment over time
  rec_rates <- cond_sim(m, target_n, m, setup_r, lambdas)
  
  rec_time <- rec_rates[nrow(rec_rates), 3]

  if(internal){
    df <- site_dist(rec_rates, int_t)
  } else {
    # If the pilot is external we generate a new set of data using the same 
    # parameters
    rec_rates_p <- cond_sim(m, n_ext, m_ext, setup_r, lambdas)
    int_t <- min(int_t, rec_rates_p[nrow(rec_rates_p), 3])
    #site_t <- min(int_t, rec_rates_p[nrow(rec_rates_p), 2])
    
    df <- site_dist(rec_rates_p, int_t)
  }
  
  n_p <- sum(df$y)
  m_p <- nrow(df)
  if(nrow(df) == 1 & df$y[1] == 0){ 
    n_p <- 0
    m_p <- 0
    r_p <- 0
  } else {
    n_p <- sum(df$y)
    m_p <- nrow(df)
    r_p <- n_p / sum(df$t)
  }
  
  # Output the interim data and the time to reach target n
  return(c(rec_time, n_p, m_p, r_p))
}

cond_sim <- function(m, target_n, m_p,
                     setup_r, lambdas){
  # Given the true parameter values, simulate recruitment of
  # target_n participants from m_p sites chosen from a set of
  # m sites
  
  # Simulate setup times
  setup_ts <- cumsum(stats::rexp(m_p, setup_r))
  end_rec <- setup_ts[m_p] + 20
  setup_order <- sample(1:m, m_p)
  
  # Get the different recruitment rates over time
  ## Each row is a recruitment period, changing when a site comes online
  ## Columns are start times, end times, and true overall recruitment rates
  rec_rates <- matrix(c(0, setup_order, 
                        0, setup_ts,
                        setup_ts, end_rec,
                        0, cumsum(lambdas[setup_order])), 
                      ncol = 4)
  
  # Simulate numbers recruited in each period and add these as a column
  ## First get the vector of expected numbers recruited in each period
  exp_rec <- (rec_rates[,4]*(rec_rates[,3] - rec_rates[,2]))[2:(m_p+1)]
  ## Now sample from a Poisson for each period
  rec_rates <- cbind(rec_rates, c(0, stats::rpois(m_p, exp_rec)))
  
  # Get time at which total target n is hit
  ## First get the period when it happens
  ## If it is not hit in the (arbitrary) 10 year period, add another period for 
  ## the remainder
  if(sum(rec_rates[,5]) < target_n){
    # Recruit to target in the final period
    rec_rates[nrow(rec_rates), 5] <- target_n - sum(rec_rates[1:m_p, 5])
  }
  
  ## Add the lambdas as a column
  rec_rates <- cbind(rec_rates, c(0, lambdas[setup_order]))
  
  fin_period <- which(cumsum(rec_rates[,5]) >= target_n)[1]
  ## Determine how many people need to arrive in that final period
  n_needed <- target_n - sum(rec_rates[1:(fin_period-1), 5])
  ## Simulate when the target is hit, which follows a gamma distribution
  rec_time <- rec_rates[fin_period, 2] + stats::rgamma(1, shape = n_needed, rate = rec_rates[fin_period, 4])
  ## Update the rec_rates summary
  rec_rates <- rec_rates[1:fin_period,]
  rec_rates[fin_period, c(3,5)] <- c(rec_time, n_needed)
  
  return(rec_rates)
}

site_dist <- function(rec_rates, int_t){
  # For a give recruitment process and interim timing, return a df listing the
  # sites, how long they were open for (up to interim), and how many were
  # recruited.
  
  ## Note how many sites are set up at interim
  m_p <- sum(rec_rates[2:nrow(rec_rates),2] <= int_t)
  if(m_p > 0){
    ## Get period (row in matrix) where interim occurs
    int_period <- utils::tail(which(rec_rates[,2] <= int_t), n=1)
    ## Start with the number recruited from all previous periods
    n_p <- sum(rec_rates[1:(int_period-1), 5])
    ## Simulate the extra number recruited in the final period, using the fact
    ## that arrivals will be uniformly distributed over the period
    n_p <- n_p +  sum(stats::runif(rec_rates[int_period, 5], rec_rates[int_period, 2], rec_rates[int_period, 3]) < int_t)
    ## Distribute these participants to sites in proportion to their expected
    ## numbers, given their true rates and times open
    n_ps <- stats::rmultinom(1, n_p, prob = (int_t- rec_rates[2:(m_p+1), 2])*
                        rec_rates[2:(m_p+1), 6])
    
    # Create a data frame storing the sites open at interim, the numbers 
    # they each recruited, and how long they were recruiting for
    df <- data.frame(y = n_ps[1:m_p],
                     c = rec_rates[2:(m_p+1), 1],
                     t = int_t - rec_rates[2:(m_p+1), 2])
  } else {
    # If no sites are recruited by interim, handle this manually
    df <- data.frame(y = 0, c = 0, t = 0)
  }
  
  return(df)
}

test_that("simulate_data returns a numeric vector of length 4", {
  set.seed(123)
  
  out <- simulate_data(
    beta_m = 2,
    beta_s = 0.3,
    v_sh = 30,
    v_r = 100,
    setup_r_a = 30,
    setup_r_b = 2.85,
    target_n = 50,
    m = 10,
    internal = TRUE,
    t = 1
  )
  
  expect_type(out, "double")
  expect_length(out, 4)
})

test_that("simulate_data returns finite, non-negative values", {
  set.seed(123)
  
  out <- simulate_data(
    beta_m = 2,
    beta_s = 0.3,
    v_sh = 30,
    v_r = 100,
    setup_r_a = 30,
    setup_r_b = 2.85,
    target_n = 50,
    m = 10,
    internal = TRUE,
    t = 1
  )
  
  expect_true(all(is.finite(out)))
  expect_true(all(out >= 0))
})

test_that("simulate_data recruitment time is positive", {
  set.seed(456)
  
  rec_time <- simulate_data(
    beta_m = 2,
    beta_s = 0.3,
    v_sh = 30,
    v_r = 100,
    setup_r_a = 30,
    setup_r_b = 2.85,
    target_n = 50,
    m = 10,
    internal = TRUE,
    t = 1
  )[1]
  
  expect_gt(rec_time, 0)
})

test_that("cond_sim returns a matrix with expected structure", {
  set.seed(123)
  
  lambdas <- rgamma(10, shape = 2, rate = 1)
  
  rec_rates <- cond_sim(
    m = 10,
    target_n = 50,
    m_p = 10,
    setup_r = 1,
    lambdas = lambdas
  )
  
  expect_true(is.matrix(rec_rates))
  expect_gte(ncol(rec_rates), 6)
  expect_gte(nrow(rec_rates), 2)
})

test_that("cond_sim reaches exactly the target sample size", {
  set.seed(456)
  
  lambdas <- rgamma(10, shape = 2, rate = 1)
  target_n <- 50
  
  rec_rates <- cond_sim(
    m = 10,
    target_n = target_n,
    m_p = 10,
    setup_r = 1,
    lambdas = lambdas
  )
  
  total_recruited <- sum(rec_rates[, 5])
  
  expect_equal(total_recruited, target_n)
})

test_that("cond_sim times are non-decreasing", {
  set.seed(789)
  
  lambdas <- rgamma(10, shape = 2, rate = 1)
  
  rec_rates <- cond_sim(
    m = 10,
    target_n = 50,
    m_p = 10,
    setup_r = 1,
    lambdas = lambdas
  )
  
  expect_true(all(diff(rec_rates[, 2]) >= 0))
  expect_true(all(diff(rec_rates[, 3]) >= 0))
})

test_that("site_dist handles zero sites open at interim", {
  rec_rates <- matrix(
    c(0, 1, 2, 1, 10, 0.5),
    ncol = 2
  )
  
  df <- site_dist(rec_rates, int_t = 0)
  
  expect_true(is.data.frame(df))
  expect_equal(nrow(df), 1)
  expect_equal(df$y[1], 0)
  expect_equal(df$t[1], 0)
})

test_that("site_dist returns sensible output when sites are open", {
  set.seed(123)
  
  lambdas <- rgamma(5, 2, 1)
  rec_rates <- cond_sim(
    m = 5,
    target_n = 20,
    m_p = 5,
    setup_r = 1,
    lambdas = lambdas
  )
  
  int_t <- rec_rates[nrow(rec_rates), 2] / 2
  df <- site_dist(rec_rates, int_t)
  
  expect_true(is.data.frame(df))
  expect_named(df, c("y", "c", "t"))
  
  expect_true(all(df$y >= 0))
  expect_true(all(df$t >= 0))
})

new_fahb_problem <- function(N, m, t, n_ext, m_ext,
                             internal,
                             rel_thr,
                             exp_T, thr,
                             so_hp_a, so_hp_b,
                             mean_rr_hp_a, mean_rr_hp_b,
                             sd_rr_hp_a, sd_rr_hp_b,
                             sims){
  
  structure(list(N=N, m=m, t=t, n_ext=n_ext, m_ext=m_ext,
                 internal=internal,
                 rel_thr=rel_thr,
                 exp_T=exp_T, thr=thr,
                 so_hp_a=so_hp_a, so_hp_b=so_hp_b,
                 mean_rr_hp_a=mean_rr_hp_a, mean_rr_hp_b = mean_rr_hp_b,
                 sd_rr_hp_a=sd_rr_hp_a, sd_rr_hp_b=sd_rr_hp_b,
                 sims=sims),
            class = "fahb_problem")
}

validate_fahb_problem <- function(y){

  return(TRUE)  
}

#' Build a `fahb_problem` object
#' 
#' Given a trial design and a set of model hyperparameters, build an object
#' of class `fahb_problem`.
#'
#' @param N target sample size.
#' @param m number of recruiting sites.
#' @param t timing of the pilot analysis in years.
#' @param n_ext number of participants to recruit to an external pilot.
#' @param m_ext number of sites to open in an external pilot.
#' @param rel_thr threshold which discriminates feasible and infeasible trials,
#'  as a multiple of the expected time to recruit.
#' @param so_hps site opening rate hyperparameters (shape and rate for a Gamma prior).
#' @param mean_rr_hps mean site recruitment rate hyperparameters (mean and sd 
#' for a lognormal prior).
#' @param sd_rr_hps variance in site recruitment rates hyperparameters (shape 
#' and rate for a Gamma prior).
#'
#' @returns an object of class `fahb_problem`
#' @export
#'
#' @examples 
#' fahb_problem()
#' 
fahb_problem <- function(N = 320, m = 20 , t = 0.5, 
                         n_ext = NULL, m_ext = NULL,
                         rel_thr = 1.2,
                         so_hps = c(30, 2.85), 
                         mean_rr_hps = c(2, 0.329), 
                         sd_rr_hps = c(30, 100)){
  # Defaults from our GUSTO example
  
  if(!is.null(n_ext)){
    if(!is.null(m_ext)){
      internal <- FALSE
      if(n_ext < 1 | m_ext < 1){
        stop("n_ext and m_ext must be >= 1")
      }
    } else {
      stop("Both n_ext and m_ext must be supplied for the external pilot case")
    }
  } else {
    internal <- TRUE
    if(!is.null(m_ext)){
      stop("Both n_ext and m_ext must be supplied for the external pilot case")
    }
  }
  
  if(t < 0){
    stop("t must be > 0")
  }
  
  # Check inputs make sense
  if(N < 1 | m < 1){
    stop("N and m must be >= 1")
  }

  if(any(c(so_hps, mean_rr_hps, sd_rr_hps) <= 0)){
    stop("All hyperparameters must be > 0")
  }
  
  # Calculate exp_T - the expected time for trial to recruit
  exp_T <- exp_rec_time(m, N, so_hps, mean_rr_hps)
  
  # Get the threshold of rec time which denotes feasibility
  thr <- rel_thr*exp_T
  
  new_fahb_problem(N, m, t, 
                   n_ext, m_ext, internal,
                   rel_thr,
                   exp_T, thr,
                   so_hp_a = so_hps[1], so_hp_b = so_hps[2],
                   mean_rr_hp_a = mean_rr_hps[1], mean_rr_hp_b = mean_rr_hps[2],
                   sd_rr_hp_a = sd_rr_hps[1], sd_rr_hp_b = sd_rr_hps[2],
                   sims = NULL)
  
  }
  
exp_rec_time <- function(m, N, so_hps, mean_rr_hps){
  
  lambda_a <- so_hps[1]; lambda_b <- so_hps[2]
  beta_m <- mean_rr_hps[1]; beta_s <- mean_rr_hps[2]
  
  # Expected rate of recruitment after all sites open
  final_rate <- exp(beta_m + beta_s^2/2)*m
  # Expected time until all sites open
  all_open <- m/(lambda_a/lambda_b)
  # Expected number recruited when all sites open
  n_0 <- final_rate*all_open/2
  if(n_0 >= N){
    exp_time <- sqrt(2*N*all_open/final_rate)
  } else {
    exp_time <- (N - (n_0 - final_rate*all_open))/final_rate
  }
  return(exp_time)
}

test_that("expected recruitment time increases with N", {
  t1 <- fahb_problem(N = 100)$exp_T
  t2 <- fahb_problem(N = 200)$exp_T
  expect_gt(t2, t1)
})

test_that("fahb_problem returns a fahb_problem object", {
  prob <- fahb_problem()
  
  expect_s3_class(prob, "fahb_problem")
  expect_type(prob, "list")
})

test_that("fahb_problem contains required fields", {
  prob <- fahb_problem()
  
  expect_named(
    prob,
    c(
      "N", "m", "t", "n_ext", "m_ext", "internal", "rel_thr",
      "exp_T", "thr",
      "so_hp_a", "so_hp_b",
      "mean_rr_hp_a", "mean_rr_hp_b",
      "sd_rr_hp_a", "sd_rr_hp_b",
      "sims"
    )
  )
})

test_that("defaults are internally consistent", {
  prob <- fahb_problem()
  
  # Derived quantities
  expect_equal(prob$thr, prob$rel_thr * prob$exp_T)
  
  # Sims should be NULL on construction
  expect_null(prob$sims)
})

test_that("N and m must be >= 1", {
  expect_error(
    fahb_problem(N = 0),
    "N and m must be >= 1"
  )
  
  expect_error(
    fahb_problem(m = 0),
    "N and m must be >= 1"
  )
})

test_that("t must be > 0", {
  expect_error(
    fahb_problem(t = -0.1),
    "t must be > 0"
  )
})

test_that("hyperparameters must be positive", {
  expect_error(
    fahb_problem(so_hps = c(-1, 1)),
    "hyperparameters must be > 0"
  )
  
  expect_error(
    fahb_problem(mean_rr_hps = c(1, -0.1)),
    "hyperparameters must be > 0"
  )
  
  expect_error(
    fahb_problem(sd_rr_hps = c(0, 1)),
    "hyperparameters must be > 0"
  )
})

test_that("changing inputs propagates correctly into the object", {
  prob <- fahb_problem(
    N = 100,
    m = 10,
    t = 0.25,
    rel_thr = 1.5,
    so_hps = c(5, 2),
    mean_rr_hps = c(1, 0.5),
    sd_rr_hps = c(10, 20)
  )
  
  expect_equal(prob$N, 100)
  expect_equal(prob$m, 10)
  expect_equal(prob$t, 0.25)
  expect_equal(prob$rel_thr, 1.5)
  
  expect_equal(prob$so_hp_a, 5)
  expect_equal(prob$so_hp_b, 2)
  
  expect_equal(prob$mean_rr_hp_a, 1)
  expect_equal(prob$mean_rr_hp_b, 0.5)
  
  expect_equal(prob$sd_rr_hp_a, 10)
  expect_equal(prob$sd_rr_hp_b, 20)
})

test_that("expected recruitment time is positive and finite", {
  prob <- fahb_problem()
  
  expect_true(is.numeric(prob$exp_T))
  expect_true(is.finite(prob$exp_T))
  expect_gt(prob$exp_T, 0)
})

test_that("forecast returns a fahb_problem object", {
  problem <- fahb_problem()
  out <- forecast(problem, n_sims = 10)
  
  expect_s3_class(out, "fahb_problem")
})

test_that("forecast adds a sims data frame", {
  problem <- fahb_problem()
  out <- forecast(problem, n_sims = 10)
  
  expect_true(is.data.frame(out$sims))
})

test_that("forecast produces the correct number of simulations", {
  problem <- fahb_problem()
  n_sims <- 25
  out <- forecast(problem, n_sims = n_sims)
  
  expect_equal(nrow(out$sims), n_sims)
})

test_that("forecast simulation output has expected columns", {
  problem <- fahb_problem()
  out <- forecast(problem, n_sims = 5)
  
  expect_named(out$sims, c("rec_T", "n_p", "m_p", "r_p"))
})

test_that("forecast simulation columns are numeric", {
  problem <- fahb_problem()
  out <- forecast(problem, n_sims = 5)
  
  expect_true(all(vapply(out$sims, is.numeric, logical(1))))
})

test_that("forecast errors if simulations already exist and overwrite is FALSE", {
  problem <- fahb_problem()
  problem <- forecast(problem, n_sims = 5)
  
  expect_error(
    forecast(problem, n_sims = 5),
    "Forecasts have been simulated already"
  )
})

test_that("forecast overwrites existing simulations when overwrite = TRUE", {
  problem <- fahb_problem()
  problem <- forecast(problem, n_sims = 5)
  
  old_sims <- problem$sims
  
  problem <- forecast(problem, n_sims = 10, overwrite = TRUE)
  
  expect_equal(nrow(problem$sims), 10)
  expect_false(identical(problem$sims, old_sims))
})

test_that("forecast does not modify problem metadata", {
  problem <- fahb_problem()
  out <- forecast(problem, n_sims = 5)
  
  expect_equal(out$N, problem$N)
  expect_equal(out$m, problem$m)
  expect_equal(out$t, problem$t)
  expect_equal(out$exp_T, problem$exp_T)
})

#' Generate probabilistic forecasts of trial recruitment
#'
#' @param problem an object of class `fahb_problem`.
#' @param n_sims number of replicates to use in the simulation.
#' @param overwrite boolean indicating if we want to overwrite any simulation data
#' currently held (defaults to FALSE).
#'
#' @returns an object of class `fahb_problem`.
#' @export
#'
#' @examples
#' problem <- fahb_problem()
#' problem <- forecast(problem, n_sims = 10^3)
#' 
forecast <- function(problem, n_sims = 10^4, overwrite = FALSE){
  
  if(!is.null(problem$sims) & !overwrite){
    stop("Forecasts have been simulated already - to discard, use rewrite = TRUE")
  }
  
  r <- t(replicate(n_sims, 
                    simulate_data(beta_m=problem$mean_rr_hp_a, beta_s=problem$mean_rr_hp_b,
                                  v_sh=problem$sd_rr_hp_a, v_r=problem$sd_rr_hp_b,
                                  setup_r_a=problem$so_hp_a, setup_r_b=problem$so_hp_b,
                                  target_n=problem$N, m=problem$m, 
                                  internal=problem$internal, t=problem$t,
                                  n_ext=problem$n_ext, m_ext=problem$m_ext)))
  r <- as.data.frame(r)
  names(r) <- c("rec_T", "n_p", "m_p", "r_p")
   
  problem$sims <- r
  return(problem)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_problem.R
\name{fahb_problem}
\alias{fahb_problem}
\title{Build a \code{fahb_problem} object}
\usage{
fahb_problem(
  N = 320,
  m = 20,
  t = 0.5,
  n_ext = NULL,
  m_ext = NULL,
  rel_thr = 1.2,
  so_hps = c(30, 2.85),
  mean_rr_hps = c(2, 0.329),
  sd_rr_hps = c(30, 100)
)
}
\arguments{
\item{N}{target sample size.}

\item{m}{number of recruiting sites.}

\item{t}{timing of the pilot analysis in years.}

\item{n_ext}{number of participants to recruit to an external pilot.}

\item{m_ext}{number of sites to open in an external pilot.}

\item{rel_thr}{threshold which discriminates feasible and infeasible trials,
as a multiple of the expected time to recruit.}

\item{so_hps}{site opening rate hyperparameters (shape and rate for a Gamma prior).}

\item{mean_rr_hps}{mean site recruitment rate hyperparameters (mean and sd
for a lognormal prior).}

\item{sd_rr_hps}{variance in site recruitment rates hyperparameters (shape
and rate for a Gamma prior).}
}
\value{
an object of class \code{fahb_problem}
}
\description{
Given a trial design and a set of model hyperparameters, build an object
of class \code{fahb_problem}.
}
\examples{
fahb_problem()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_analysis.R
\name{plot.fahb_analysis}
\alias{plot.fahb_analysis}
\title{Plot posterior distributions from a \code{fahb} analysis}
\usage{
\method{plot}{fahb_analysis}(x, ...)
}
\arguments{
\item{x}{object of class \code{fahb_analysis} as produced by \verb{fahb_analysis().}}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
Takes an object of class \code{fahb_analysis} and plots the posterior
distributions of the predicted time for the trial to recruit and of the three
model parameters.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_design.R
\name{plot.fahb_design}
\alias{plot.fahb_design}
\title{Plot operating characteristics of fahb designs}
\usage{
\method{plot}{fahb_design}(x, ...)
}
\arguments{
\item{x}{object of class \code{fahb_design} as produced by \verb{fahb_design().}}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
Takes an object of class \code{fahb_design} and plots the estimated operating
characteristics of decision rules - based on standard progression criteria,
an approximate Bayesian analysis, or both.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_priors.R
\name{check_priors}
\alias{check_priors}
\title{Plot prior distributions}
\usage{
check_priors(problem)
}
\arguments{
\item{problem}{an object of class \code{fahb_problem}.}
}
\value{
a list of \code{ggplot2} plots.
}
\description{
Given the hyperparameters encoded in a \code{fahb_problem} object, return plots of
the three prior distributions and a plot of the prior predictive distribution
of the recruitment rate at a randomly selected site.
}
\examples{
problem <- fahb_problem()
check_priors(problem)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forecast.R
\name{forecast}
\alias{forecast}
\title{Generate probabilistic forecasts of trial recruitment}
\usage{
forecast(problem, n_sims = 10^4, overwrite = FALSE)
}
\arguments{
\item{problem}{an object of class \code{fahb_problem}.}

\item{n_sims}{number of replicates to use in the simulation.}

\item{overwrite}{boolean indicating if we want to overwrite any simulation data
currently held (defaults to FALSE).}
}
\value{
an object of class \code{fahb_problem}.
}
\description{
Generate probabilistic forecasts of trial recruitment
}
\examples{
problem <- fahb_problem()
problem <- forecast(problem, n_sims = 10^3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_analysis.R
\name{fahb_analysis}
\alias{fahb_analysis}
\title{Build a \code{fahb} analysis object}
\usage{
fahb_analysis(n_pilot, t_pilot, problem, site_t = NULL, bayes_model = NULL)
}
\arguments{
\item{n_pilot}{integer vector of numbers recruited at each open site.}

\item{t_pilot}{numeric vector of time (in years) each site has been open.}

\item{problem}{object of class \code{fahb_problem}.}

\item{site_t}{In the case of an external pilot, the time taken for all
pilot sites to open.}

\item{bayes_model}{optional object of class \code{brmsfit} which will be used in
the Bayesian analysis via \code{brms::update()} to avoid compiling a new model.}
}
\value{
An object of class \code{fahb_analysis}.
}
\description{
Given a \code{fahb_problem} object calculate summary statistics which can inform
progression decisions. These include both standard progression criteria
statistics, and the expectation of the posterior predictive distribution of
the time until the trial recruits.
}
\examples{
## Example illustrating a full analysis workflow
## (Not run on CRAN due to Bayesian model fitting)

\donttest{
problem <- fahb_problem()
problem <- forecast(problem)

## Pilot trial data
n_pilot <- c(3, 5, 2)
t_pilot <- c(0.5, 0.6, 0.4)

analysis <- fahb_analysis(
  n_pilot = n_pilot,
  t_pilot = t_pilot,
  problem = problem
)

print(analysis)
plot(analysis)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_design.R
\name{print.fahb_design}
\alias{print.fahb_design}
\title{Print a fahb design object}
\usage{
\method{print}{fahb_design}(x, coarse = TRUE, ...)
}
\arguments{
\item{x}{object of class \code{fahb_design} as produced by \code{fahb_design()}.}

\item{coarse}{binary indicator that only a coarse subset of all decision
rules should be printed. Defaults to TRUE.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
The default print method for a \code{fahb_design} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_analysis.R
\name{print.fahb_analysis}
\alias{print.fahb_analysis}
\title{Print a \code{fahb} analysis object}
\usage{
\method{print}{fahb_analysis}(x, ...)
}
\arguments{
\item{x}{object of class \code{fahb_analysis} as produced by \code{fahb_analysis()}.}

\item{...}{further arguments passed to or from other methods.}
}
\value{
no return value, called for side effects.
}
\description{
The default print method for a \code{fahb_analysis} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb_design.R
\name{fahb_design}
\alias{fahb_design}
\title{Build a \code{fahb} design object}
\usage{
fahb_design(problem, quietly = TRUE)
}
\arguments{
\item{problem}{an object of class \code{fahb_problem}.}

\item{quietly}{if this argument is set to FASLE then information about which
steps have been completed will be printed to the console. Defaults to TRUE.}
}
\value{
an object of class \code{fahb_design}.
}
\description{
Given a \code{fahb_problem} object, find efficient progression decision rules.
These can include rules of the standard "progression criteria form", or rules
based on a Bayesian analysis of the pilot trial data, or both.
}
\examples{
problem <- forecast(fahb_problem(), n_sims = 500)
fahb_design(problem)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fahb-package.R
\docType{package}
\name{fahb-package}
\alias{fahb}
\alias{fahb-package}
\title{fahb: Feasibility Assessment with Hierarchical Bayes}
\description{
Tools for simulation, design, and analysis of internal pilot trials
using frequentist and Bayesian operating characteristics.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://dtwilson.github.io/fahb/}
  \item \url{https://github.com/DTWilson/fahb}
  \item Report bugs at \url{https://github.com/DTWilson/fahb/issues}
}

}
\author{
\strong{Maintainer}: Duncan Wilson \email{d.t.wilson@leeds.ac.uk} (\href{https://orcid.org/0000-0001-7949-8718}{ORCID}) [copyright holder]

}
\keyword{internal}
